var prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

function stig_apply() {
    var ret = confirm("Do you really want to apply the DISA STIG settings?  This may break things.");
    if (ret) {
        stig_settings();
        stig_audit();
    }
}

function stig_settings() {
    // DTBG010
    if (!root_cert_exists()) {
        install_certs();
    }
    // DTBG020
    prefManager.setBoolPref("security.enable_ssl3", false);
    // DTBG050
    prefManager.setCharPref("security.default_personal_cert", "Ask Every Time");
    // DTBG105
    prefManager.setBoolPref("network.protocol-handler.external.shell", false);
    // DTBF110
    // TODO I'm pretty sure all these mime types are not right, also update for audit
    prefManager.setCharPref("plugin.disable_full_page_plugin_for_types", "application/pdf,application/doc,application/xls,application/bat,application/ppt,application/mdb,application/mde,application/vnd.fdf,application/vnd.adobe.xfdf,application/lsl,application/lso,application/lss,application/iqy,application/rqy,application/xlk,application/pot,application/pps,application/dot,application/wbk,application/ps,application/eps,application/wch,application/wcm,application/wbi,application/wb1,application/wb3,application/rtf,application/wch,application/wcm,application/ad,application/adp");
    // TODO DTBF120: FireFox plug-in for ActiveX controls is installed.
    // TODO DTBF100: Firefox automatically executes or downloads MIME types which are not authorized for auto-download.
    // Not configurable here: DTBF070: Firefox required security preferences can not be changed by user.
    // DTBG140
    prefManager.setBoolPref("browser.formfill.enable", false);
    // DTBG150
    prefManager.setBoolPref("signon.prefillForms", false);
    prefManager.setBoolPref("signon.autofillForms", false);
    // DTBG160
    prefManager.setBoolPref("signon.rememberSignons", false);
    // DTBG170
    prefManager.setBoolPref("privacy.sanitize.promptOnSanitize", false);
    prefManager.setBoolPref("privacy.sanitize.sanitizeOnShutdown", true);
    // DTBG180
    prefManager.setBoolPref("dom.disable_window_open_feature.status", true);
    // DTBG181
    prefManager.setBoolPref("dom.disable_window_move_resize", true);
    // DTBG010
    prefManager.setBoolPref("security.enable_ssl2", false);
    // DTBG030
    prefManager.setBoolPref("security.enable_tls", true);
    // DTBG182
    prefManager.setBoolPref("dom.disable_window_flip", true);
    // DTBG183
    prefManager.setBoolPref("dom.event.contextmenu.enabled", false);
    // DTBG184
    prefManager.setBoolPref("dom.disable_window_status_change", true);
    // DTBG185
    prefManager.setBoolPref("dom.disable_window_open_feature.status", true);
    // DTBG130
    prefManager.setBoolPref("security.warn_leaving_secure", true);
    // DTBG017
    prefManager.setCharPref("browser.startup.homepage", "about:blank");
    // DTBG080
    prefManager.setBoolPref("app.update.enabled", false);
    // DTBG090
    prefManager.setBoolPref("extensions.update.enabled", false);
    // DTBG085
    prefManager.setBoolPref("browser.search.update", false);
}

function stig_audit() {
    var audit_list = [];
    // check root sha1 sum here
    if (!root_cert_exists()) {
        audit_list.push("DTBG010: The DOD Root Certificate is not installed.");
    }
    if (prefManager.getBoolPref("security.enable_ssl3")) {
        audit_list.push("DTBG020: Firefox is configured to allow use of SSL 3.0.");
    }
    if (!prefManager.getCharPref("security.default_personal_cert", "Ask Every Time")) {
        audit_list.push("DTBF050: FireFox is not configured to ask which certificate to present to a web site when a certificate is required.");
    }
    var mime_type_list = [];
    var exts = ["application/pdf","application/doc","application/xls","application/bat","application/ppt","application/mdb","application/mde","application/vnd.fdf","application/vnd.adobe.xfdf","application/lsl","application/lso","application/lss","application/iqy","application/rqy","application/xlk","application/pot","application/pps","application/dot","application/wbk","application/ps","application/eps","application/wch","application/wcm","application/wbi","application/wb1","application/wb3","application/rtf","application/wch","application/wcm","application/ad","application/adp"];
    for (var i = 0; i < exts.length; i++) {
        var ext = exts[i];
        var app = mime_type_enabled(ext);
        if (app) {
            mime_type_list.push(ext);
        }
    }
    if (mime_type_list.length > 0) {
        audit_list.push("DTBF110: Firefox not configured to prompt user before download and opening for required file types. (Count: " + mime_type_list.length + ")");
    }

    if (prefManager.getBoolPref("network.protocol-handler.external.shell")) {
        audit_list.push("DTBF105: Network shell protocol is enabled in FireFox.");
    }

    // DTBF100: Firefox not configured to prompt user before download and opening for required file types.
    var exts = ["hta", "jse", "js", "mocha", "shs", "vbe", "vbs", "sct", "wsc"];
    for (var i = 0; i < exts.length; i++) {
        var ext = exts[i];
        var app = extension_enabled(ext);
        if (app != "") {
            audit_list.push("DTBF100: Firefox not configured to prompt user before download and opening for ." + ext + ".");
        }
    }
    if (activex_enabled()) {
        audit_list.push("DTBF120: FireFox plug-in for ActiveX controls is installed.");
    }
    if (prefManager.getBoolPref("browser.formfill.enable")) {
        audit_list.push("DTBF140: Firefox formfill assistance option is disabled.");
    }

    var val = true;
    try {
       var val = prefManager.getBoolPref("signon.prefillForms");
    } catch (e) {};
    val = val || prefManager.getBoolPref("signon.autofillForms");
    if (val) {
        audit_list.push("DTBF150: Firefox is configured to autofill passwords.");
    }

    if (prefManager.getBoolPref("signon.rememberSignons")) {
        audit_list.push("DTBF160: FireFox is configured to use a password store with or without a master password.");
    }

    var val = true;
    try {
       var val = prefManager.getBoolPref("privacy.sanitize.promptOnSanitize");
    } catch (e) {};
    val = val || !prefManager.getBoolPref("privacy.sanitize.sanitizeOnShutdown");
    if (val) {
        audit_list.push("DTBF170: Firefox does not clear cookies upon closing.");
    }

    if (!prefManager.getBoolPref("dom.disable_window_open_feature.status")) {
        audit_list.push("DTBF180: FireFox is not configured to block pop-up windows.");
    }
    if (!prefManager.getBoolPref("dom.disable_window_move_resize")) {
        audit_list.push("DTBF181: FireFox is configured to allow JavaScript to move or resize windows.");
    }
    try {
    if (prefManager.getBoolPref("security.enable_ssl2")) {
        audit_list.push("DTBF010: The Firefox SSLV2 parameter is configured to allow use of SSL 2.0.");
    }
    } catch (e) {};
    if (!prefManager.getBoolPref("security.enable_tls")) {
        audit_list.push("DTBF030: Firefox is not configured to allow use of TLS 1.0.");
    }
    if (!prefManager.getBoolPref("dom.disable_window_flip")) {
        audit_list.push("DTBF182: Firefox is configured to allow JavaScript to raise or lower windows.");
    }
    if (prefManager.getBoolPref("dom.event.contextmenu.enabled")) {
        audit_list.push("DTBF183: Firefox is configured to allow JavaScript to disable or replace context menus.");
    }
    if (!prefManager.getBoolPref("dom.disable_window_status_change")) {
        audit_list.push("DTBF184: Firefox is configured to allow JavaScript to hide or change the status bar.");
    }
    if (!prefManager.getBoolPref("dom.disable_window_open_feature.status")) {
        audit_list.push("DTBF185: Firefox is configured to allow JavaScript to change the status bar text.");
    }
    try {
    if (!prefManager.getBoolPref("security.warn_leaving_secure")) {
        audit_list.push("DTBF130: Firefox is not configured to provide warnings when a user switches from a secure (SSL-enabled) to a non-secure page.");
    }
    } catch (e) {};
    if (prefManager.getCharPref("browser.startup.homepage") != "about:blank" && !endsWith(prefManager.getCharPref("browser.startup.homepage"),".mil")) {
        audit_list.push("DTBF017: The Firefox browser home page is not set to blank or a trusted site.");
    }
    if (!stig_is_gteq("3")) {
        audit_list.push("DTBF003: Installed version of Firefox not supported (less than 3).");
    }
    if (prefManager.getBoolPref("app.update.enabled")) {
        audit_list.push("DTBF080: Firefox application is set to auto-update.");
    }
    if (prefManager.getBoolPref("extensions.update.enabled")) {
        audit_list.push("DTBF090: Firefox automatically updates installed add-ons and plugins.");
    }

    var unlocked_list = [];
    var locked_prefs = ["security.enable_ssl3", "security.enable_ssl2","security.default_personal_cert", "network.protocol-handler.external.shell","plugin.disable_full_page_plugin_for_types", "browser.formfill.enable", "signon.prefillForms", "signon.rememberSignons", "privacy.sanitize.sanitizeOnShutdown", "privacy.sanitize.promptOnSanitize", "dom.disable_window_open_feature.status", "dom.disable_window_move_resize", "security.enable_tls", "dom.disable_window_flip", "dom.event.contextmenu.enabled", "dom.disable_window_status_change", "dom.disable_window_open_feature.status", "security.warn_leaving_secure", "browser.startup.homepage", "app.update.enabled", "extensions.update.enabled", "browser.search.update"]; 

    for (var i = 0; i < locked_prefs.length; i++) {
        var prefname = locked_prefs[i];
        if (!prefManager.prefIsLocked(prefname)) {
            unlocked_list.push(prefname);
        }
    }
    if (unlocked_list.length > 0) {
        audit_list.push("DTBF070: Firefox required security preferences can not be changed by user. (Count: " + unlocked_list.length + ")");
    }


    if (prefManager.getBoolPref("browser.search.update")) {
        audit_list.push("DTBF085: Firefox automatically checks for updated version of installed Search plugins.");
    }

    var audit_string = "The following findings exist:\r\n\r\n" + audit_list.join("\r\n");
    if (audit_list.length > 0) {
        alert(audit_string);
    } else {
        alert("Everything looks good!");
    }
}

////// Test support functions

function endsWith(str, suffix) {
    return str.indexOf(suffix, str.length - suffix.length) !== -1;
}


function stig_is_gteq(ver) {
    // check if app version is greater than or equal to
    const id = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
    if(appInfo.ID == id) {
        // running under id
        var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator);
        if(versionChecker.compare(appInfo.version, ver) >= 0) {
            return true;
        }
    }
    return false;
}

function root_cert_exists() {
	return check_fingerprint("DoD Root CA 2 - U.S. Government", "8C:94:1B:34:EA:1E:A6:ED:9A:E2:BC:54:CF:68:72:52:B4:C9:B5:61");
}

function check_fingerprint(name, sha1) {
        var certdb1 = Components.classes["@mozilla.org/security/x509certdb;1"].getService(Components.interfaces.nsIX509CertDB);
        try {
            var cert = certdb1.findCertByNickname(null, name);
        } catch (e) {
            //alert("root cert not found");
            return false;
        };
        if (cert == null) {
            //alert("root cert not found 2");
            return false;
        }
        if (cert.sha1Fingerprint == sha1) {
            return true;
        }
	return false;    
}

function extension_enabled(ext) {
    for (var i = 0; i < navigator.mimeTypes.length; i++) {
        var suffixes = navigator.mimeTypes[i].suffixes.split(",");
        for (var j = 0; j < suffixes.length; j++) {
            suffix = suffixes[j];
            if (suffix.toLowerCase() == ext.toLowerCase()) {
                return navigator.mimeTypes[i].description;
            }
        }
    }
    return "";
}

function mime_type_enabled(ext) {
    var suffixes = prefManager.getCharPref("plugin.disable_full_page_plugin_for_types").split(",");
    for (var j = 0; j < suffixes.length; j++) {
            suffix = suffixes[j];
            if (suffix.toLowerCase() == ext.toLowerCase()) {
                return false;
            }
    }
    return true;
}


function activex_enabled() {
    return plugin_enabled("application/oleobject");
}

function plugin_enabled(mimetype) {
    nse = ""; for (var i=0;i<navigator.mimeTypes.length;i++) nse += navigator.mimeTypes[i].type.toLowerCase();
    if (nse.indexOf(mimetype) == -1) {
        return false;
    }
    if (navigator.mimeTypes[mimetype] != null) {
        return true;
    }
    return false;
}